@@ -107,11 +107,14 @@ public:
 
 	KafkaList& operator= (const KafkaList& copy)
 	{
-		this->~KafkaList();
-		this->ref = copy.ref;
-		++*this->ref;
-		this->t_list = copy.t_list;
-		this->curpos = copy.curpos;
+		if (this != &copy)
+		{
+			this->~KafkaList();
+			this->ref = copy.ref;
+			++*this->ref;
+			this->t_list = copy.t_list;
+			this->curpos = copy.curpos;
+		}
 		return *this;
 	}
 
@@ -227,10 +230,13 @@ public:
 
 	KafkaMap& operator= (const KafkaMap& copy)
 	{
-		this->~KafkaMap();
-		this->ref = copy.ref;
-		++*this->ref;
-		this->t_map = copy.t_map;
+		if (this != &copy)
+		{
+			this->~KafkaMap();
+			this->ref = copy.ref;
+			++*this->ref;
+			this->t_map = copy.t_map;
+		}
 		return *this;
 	}
 
@@ -621,10 +627,13 @@ public:
 
 	KafkaConfig& operator= (const KafkaConfig& copy)
 	{
-		this->~KafkaConfig();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		++*this->ref;
+		if (this != &copy)
+		{
+			this->~KafkaConfig();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+		}
 		return *this;
 	}
 
@@ -633,7 +642,6 @@ public:
 private:
 	kafka_config_t *ptr;
 	std::atomic<int> *ref;
-	std::string sasl_buf;
 };
 
 class KafkaRecord
@@ -725,10 +733,13 @@ public:
 
 	KafkaRecord& operator= (KafkaRecord& copy)
 	{
-		this->~KafkaRecord();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		++*this->ref;
+		if (this != &copy)
+		{
+			this->~KafkaRecord();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+		}
 		return *this;
 	}
 
@@ -798,7 +809,12 @@ public:
 	long long get_low_watermark() const { return this->ptr->low_watermark; }
 	void set_low_watermark(long long offset) { this->ptr->low_watermark = offset; }
 
-	bool reach_high_watermark() const { return this->ptr->offset == this->ptr->high_watermark; }
+	void clear_records()
+	{
+		INIT_LIST_HEAD(&this->ptr->record_list);
+		this->curpos = &this->ptr->record_list;
+		this->startpos = this->endpos = this->curpos;
+	}
 
 public:
 	KafkaToppar()
@@ -852,13 +868,16 @@ public:
 
 	KafkaToppar& operator= (KafkaToppar& copy)
 	{
-		this->~KafkaToppar();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		++*this->ref;
-		this->curpos = copy.curpos;
-		this->startpos = copy.startpos;
-		this->endpos = copy.endpos;
+		if (this != &copy)
+		{
+			this->~KafkaToppar();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+			this->curpos = copy.curpos;
+			this->startpos = copy.startpos;
+			this->endpos = copy.endpos;
+		}
 		return *this;
 	}
 
@@ -936,6 +955,19 @@ public:
 		this->curpos = this->curpos->prev;
 	}
 
+	KafkaRecord *get_tail_record()
+	{
+		if (&this->ptr->record_list != this->ptr->record_list.prev)
+		{
+			return (KafkaRecord *)list_entry(this->ptr->record_list.prev,
+					KafkaRecord, list);
+		}
+		else
+		{
+			return NULL;
+		}
+	}
+
 private:
 	struct list_head list;
 	kafka_topic_partition_t *ptr;
@@ -1040,11 +1072,14 @@ public:
 
 	KafkaBroker& operator= (const KafkaBroker& copy)
 	{
-		this->~KafkaBroker();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		if (this->ref)
-			++*this->ref;
+		if (this != &copy)
+		{
+			this->~KafkaBroker();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			if (this->ref)
+				++*this->ref;
+		}
 
 		return *this;
 	}
@@ -1201,10 +1236,13 @@ public:
 
 	KafkaMeta& operator= (KafkaMeta& copy)
 	{
-		this->~KafkaMeta();
-		this->ptr = copy.ptr;
-		this->ref = copy.ref;
-		++*this->ref;
+		if (this != &copy)
+		{
+			this->~KafkaMeta();
+			this->ptr = copy.ptr;
+			this->ref = copy.ref;
+			++*this->ref;
+		}
 		return *this;
 	}
 
@@ -1391,10 +1429,9 @@ public:
 		return this->coordinator;
 	}
 
-	int run_assignor(KafkaMetaList *meta_list, KafkaMetaList *alien_meta_list, 
-					 const char *protocol_name);
+	int run_assignor(KafkaMetaList *meta_list, const char *protocol_name);
 
-	void add_subscriber(KafkaMetaList *meta_list, 
+	void add_subscriber(KafkaMetaList *meta_list,
 						std::vector<KafkaMetaSubscriber> *subscribers);
 
 	static int kafka_range_assignor(kafka_member_t **members,
