@@ -17,9 +17,9 @@
 */
 
 #include <arpa/inet.h>
-#include <stdint.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 #include <sstream>
 #include <utility>
@@ -41,7 +41,12 @@
 namespace protocol
 {
 
-#define CHECK_RET(exp)	if (exp < 0) return exp
+#define CHECK_RET(exp) \
+do { \
+	int tmp = exp; \
+	if (tmp < 0) \
+		return tmp; \
+} while (0)
 
 #ifndef htonll
 static uint64_t htonll(uint64_t x)
@@ -148,14 +153,6 @@ static size_t append_nullable_string(std::string& buf, const char *str, size_t l
 		return append_string(buf, str, len);
 }
 
-static size_t append_nullable_string(std::string& buf, const char *str)
-{
-	if (!str)
-		return append_nullable_string(buf, str, 0);
-	else
-		return append_nullable_string(buf, str, strlen(str));
-}
-
 static size_t append_string_raw(void **buf, const char *str, size_t len)
 {
 	memcpy(*buf, str, len);
@@ -301,7 +298,7 @@ static int parse_bytes(void **buf, size_t *size,
 
 static int parse_varint_u64(void **buf, size_t *size, uint64_t *val);
 
-static inline int parse_varint_i64(void **buf, size_t *size, int64_t *val)
+static int parse_varint_i64(void **buf, size_t *size, int64_t *val)
 {
 	uint64_t n;
 	int ret = parse_varint_u64(buf, size, &n);
@@ -312,7 +309,7 @@ static inline int parse_varint_i64(void **buf, size_t *size, int64_t *val)
 	return ret;
 }
 
-static inline int parse_varint_i32(void **buf, size_t *size, int32_t *val)
+static int parse_varint_i32(void **buf, size_t *size, int32_t *val)
 {
 	int64_t v = 0;
 
@@ -323,7 +320,7 @@ static inline int parse_varint_i32(void **buf, size_t *size, int32_t *val)
 	return 0;
 }
 
-static constexpr LZ4F_preferences_t kPrefs =
+static const LZ4F_preferences_t kPrefs =
 {
 	.frameInfo = {LZ4F_default, LZ4F_blockIndependent, },
 	.compressionLevel = 0,
@@ -539,25 +536,25 @@ static int kafka_snappy_java_uncompress(const char *inbuf, size_t inlen, KafkaBl
 
 	for (int pass = 1; pass <= 2; pass++)
 	{
-		ssize_t of = 0;
-		ssize_t uof = 0;
+		ssize_t off = 0;
+		ssize_t uoff = 0;
 
-		while (of + 4 <= (ssize_t)inlen)
+		while (off + 4 <= (ssize_t)inlen)
 		{
 			uint32_t clen;
 			size_t ulen;
 
-			memcpy(&clen, inbuf+of, 4);
+			memcpy(&clen, inbuf + off, 4);
 			clen = ntohl(clen);
-			of += 4;
+			off += 4;
 
-			if (clen > inlen - of)
+			if (clen > inlen - off)
 			{
 				errno = EBADMSG;
 				return -1;
 			}
 
-			if (snappy_uncompressed_length(inbuf+of, clen, &ulen) != SNAPPY_OK)
+			if (snappy_uncompressed_length(inbuf + off, clen, &ulen) != SNAPPY_OK)
 			{
 				errno = EBADMSG;
 				return -1;
@@ -565,24 +562,24 @@ static int kafka_snappy_java_uncompress(const char *inbuf, size_t inlen, KafkaBl
 
 			if (pass == 1)
 			{
-				of+= clen;
-				uof += ulen;
+				off += clen;
+				uoff += ulen;
 				continue;
 			}
 
-			size_t n = block->get_len() - uof;
+			size_t n = block->get_len() - uoff;
 
-			if (snappy_uncompress(inbuf+of, clen, obuf+uof, &n) != SNAPPY_OK)
+			if (snappy_uncompress(inbuf + off, clen, obuf + uoff, &n) != SNAPPY_OK)
 			{
 				errno = EBADMSG;
 				return -1;
 			}
 
-			of+= clen;
-			uof += ulen;
+			off += clen;
+			uoff += ulen;
 		}
 
-		if (of != (ssize_t)inlen)
+		if (off != (ssize_t)inlen)
 		{
 			errno = EBADMSG;
 			return -1;
@@ -590,19 +587,19 @@ static int kafka_snappy_java_uncompress(const char *inbuf, size_t inlen, KafkaBl
 
 		if (pass == 1)
 		{
-			if (uof <= 0)
+			if (uoff <= 0)
 			{
 				errno = EBADMSG;
 				return -1;
 			}
 
-			if (!block->allocate(uof))
+			if (!block->allocate(uoff))
 				return -1;
 
 			obuf = (char *)block->get_block();
 		}
 		else
-			block->set_len(uof);
+			block->set_len(uoff);
 	}
 
 	return 0;
@@ -612,10 +609,10 @@ static int snappy_decompress(void *buf, size_t n, KafkaBlock *block)
 {
 	const char *inbuf = (const char *)buf;
 	size_t inlen = n;
-	static constexpr unsigned char snappy_java_magic[] = {
+	static const unsigned char snappy_java_magic[] = {
 		0x82, 'S','N','A','P','P','Y', 0
 	};
-	static constexpr size_t snappy_java_hdrlen = 8 + 4 + 4;
+	static const size_t snappy_java_hdrlen = 8 + 4 + 4;
 
 	if (!memcmp(buf, snappy_java_magic, 8))
 	{
@@ -648,7 +645,7 @@ static int lz4_decompress(void *buf, size_t n, KafkaBlock *block)
 	LZ4F_decompressionContext_t dctx;
 	LZ4F_frameInfo_t fi;
 	size_t in_sz, out_sz;
-	size_t in_of, out_of;
+	size_t in_off, out_off;
 	size_t r;
 	size_t uncompressed_size;
 	size_t outlen;
@@ -687,14 +684,14 @@ static int lz4_decompress(void *buf, size_t n, KafkaBlock *block)
 
 	out = (char *)block->get_block();
 	outlen = block->get_len();
-	in_of = in_sz;
-	out_of = 0;
-	while (in_of < inlen)
+	in_off = in_sz;
+	out_off = 0;
+	while (in_off < inlen)
 	{
-		out_sz = outlen - out_of;
-		in_sz = inlen - in_of;
-		r = LZ4F_decompress(dctx, out + out_of, &out_sz,
-							inbuf + in_of, &in_sz, NULL);
+		out_sz = outlen - out_off;
+		in_sz = inlen - in_off;
+		r = LZ4F_decompress(dctx, out + out_off, &out_sz,
+							inbuf + in_off, &in_sz, NULL);
 		if (LZ4F_isError(r))
 		{
 			code = LZ4F_freeDecompressionContext(dctx);
@@ -702,19 +699,19 @@ static int lz4_decompress(void *buf, size_t n, KafkaBlock *block)
 			return -1;
 		}
 
-		if (!(out_of + out_sz <= outlen && in_of + in_sz <= inlen))
+		if (!(out_off + out_sz <= outlen && in_off + in_sz <= inlen))
 		{
 			code = LZ4F_freeDecompressionContext(dctx);
 			errno = EBADMSG;
 			return -1;
 		}
 
-		out_of += out_sz;
-		in_of += in_sz;
+		out_off += out_sz;
+		in_off += in_sz;
 		if (r == 0)
 			break;
 
-		if (out_of == outlen)
+		if (out_off == outlen)
 		{
 			size_t extra = outlen * 3 / 4;
 
@@ -731,7 +728,7 @@ static int lz4_decompress(void *buf, size_t n, KafkaBlock *block)
 	}
 
 
-	if (in_of < inlen)
+	if (in_off < inlen)
 	{
 		code = LZ4F_freeDecompressionContext(dctx);
 		errno = EBADMSG;
@@ -991,7 +988,7 @@ static int parse_string(void **buf, size_t *size, std::string& str)
 {
 	if (*size >= 2)
 	{
-		short len;
+		int16_t len;
 
 		if (parse_i16(buf, size, &len) >= 0)
 		{
@@ -1024,7 +1021,7 @@ static int parse_string(void **buf, size_t *size, char **str)
 {
 	if (*size >= 2)
 	{
-		short len;
+		int16_t len;
 
 		if (parse_i16(buf, size, &len) >= 0)
 		{
@@ -1070,7 +1067,7 @@ static int parse_bytes(void **buf, size_t *size, std::string& str)
 {
 	if (*size >= 4)
 	{
-		int len;
+		int32_t len;
 
 		if (parse_i32(buf, size, &len) >= 0)
 		{
@@ -1102,7 +1099,7 @@ static int parse_bytes(void **buf, size_t *size,
 {
 	if (*size >= 4)
 	{
-		int len;
+		int32_t len;
 
 		if (parse_i32(buf, size, &len) >= 0)
 		{
@@ -1132,7 +1129,7 @@ static int parse_bytes(void **buf, size_t *size,
 
 static int parse_varint_u64(void **buf, size_t *size, uint64_t *val)
 {
-	size_t of = 0;
+	size_t off = 0;
 	uint64_t num = 0;
 	int shift = 0;
 	size_t org_size = *size;
@@ -1146,13 +1143,13 @@ static int parse_varint_u64(void **buf, size_t *size, uint64_t *val)
 			return -1; /* Underflow */
 		}
 
-		num |= (uint64_t)(((char *)(*buf))[(int)of] & 0x7f) << shift;
+		num |= (uint64_t)(((char *)(*buf))[(int)off] & 0x7f) << shift;
 		shift += 7;
-	} while (((char *)(*buf))[(int)of++] & 0x80);
+	} while (((char *)(*buf))[(int)off++] & 0x80);
 
 	*val = num;
-	*buf = (char *)(*buf) + of;
-	*size -= of;
+	*buf = (char *)(*buf) + off;
+	*size -= off;
 	return 0;
 }
 
@@ -1163,8 +1160,8 @@ int KafkaMessage::parse_message_set(void **buf, size_t *size,
 									KafkaToppar *toppar)
 {
 	int64_t offset;
-	int message_size;
-	int crc;
+	int32_t message_size;
+	int32_t crc;
 
 	if (parse_i64(buf, size, &offset) < 0)
 		return -1;
@@ -1180,7 +1177,7 @@ int KafkaMessage::parse_message_set(void **buf, size_t *size,
 
 	if (check_crcs)
 	{
-		int crc_32 = crc32(0, NULL, 0);
+		int32_t crc_32 = crc32(0, NULL, 0);
 		crc_32 = crc32(crc_32, (Bytef *)*buf, message_size - 4);
 		if (crc_32 != crc)
 		{
@@ -1198,9 +1195,9 @@ int KafkaMessage::parse_message_set(void **buf, size_t *size,
 	if (parse_i8(buf, size, &attributes) < 0)
 		return -1;
 
-	int64_t timestamp;
+	int64_t timestamp = -1;
 	if (msg_vers == 1 && parse_i64(buf, size, &timestamp) < 0)
-			return -1;
+		return -1;
 
 	void *key;
 	size_t key_len;
@@ -1328,9 +1325,9 @@ int KafkaMessage::parse_message_record(void **buf, size_t *size,
 {
 	int64_t length;
 	int8_t attributes;
-	long long timestamp_delta = 0;
-	long long offset_delta = 0;
-	int hdr_size;
+	int64_t timestamp_delta;
+	int64_t offset_delta;
+	int32_t hdr_size;
 
 	if (parse_varint_i64(buf, size, &length) < 0)
 		return -1;
@@ -1338,10 +1335,10 @@ int KafkaMessage::parse_message_record(void **buf, size_t *size,
 	if (parse_i8(buf, size, &attributes) < 0)
 		return -1;
 
-	if (parse_varint_i64(buf, size, (int64_t *)&timestamp_delta) < 0)
+	if (parse_varint_i64(buf, size, &timestamp_delta) < 0)
 		return -1;
 
-	if (parse_varint_i64(buf, size, (int64_t *)&offset_delta) < 0)
+	if (parse_varint_i64(buf, size, &offset_delta) < 0)
 		return -1;
 
 	record->timestamp += timestamp_delta;
@@ -1617,6 +1614,7 @@ KafkaMessage::KafkaMessage()
 	kafka_parser_init(this->parser);
 	this->stream = new EncodeStream;
 	this->api_type = Kafka_Unknown;
+	this->correlation_id = 0;
 	this->cur_size = 0;
 }
 
@@ -1760,7 +1758,7 @@ int KafkaMessage::encode_head()
 	append_i32(this->headbuf, 0);
 	append_i16(this->headbuf, this->api_type);
 	append_i16(this->headbuf, this->api_version);
-	append_i32(this->headbuf, 0);
+	append_i32(this->headbuf, this->correlation_id);
 	append_string(this->headbuf, this->config.get_client_id());
 
 	return 0;
@@ -2168,7 +2166,8 @@ int KafkaRequest::encode_produce(struct iovec vectors[], int max)
 
 			if (ret < 0)
 				return -1;
-			else if (ret > 0)
+
+			if (ret > 0)
 			{
 				toppar->record_rollback();
 				toppar->save_record_endpos();
@@ -2804,11 +2803,13 @@ int KafkaResponse::parse_response()
 
 	void *buf = this->parser->msgbuf;
 	size_t size = this->parser->message_size;
-	int correlation_id;
+	int32_t correlation_id;
 
 	if (parse_i32(&buf, &size, &correlation_id) < 0)
 		return -1;
 
+	this->correlation_id = correlation_id;
+
 	int ret = it->second(&buf, &size);
 
 	if (ret < 0)
@@ -2827,7 +2828,7 @@ static int kafka_meta_parse_broker(void **buf, size_t *size,
 								   int api_version,
 								   KafkaBrokerList *broker_list)
 {
-	int broker_cnt;
+	int32_t broker_cnt;
 
 	CHECK_RET(parse_i32(buf, size, &broker_cnt));
 
@@ -2892,6 +2893,7 @@ static bool kafka_broker_get_leader(int leader_id, KafkaBrokerList *broker_list,
 			if (host)
 			{
 				char *rack;
+
 				if (broker->rack)
 					rack = strdup(broker->rack);
 
@@ -2922,10 +2924,10 @@ static int kafka_meta_parse_partition(void **buf, size_t *size,
 									  KafkaMeta *meta,
 									  KafkaBrokerList *broker_list)
 {
-	int i, j;
-	int leader_id;
-	int replica_cnt, isr_cnt;
-	int partition_cnt = 0;
+	int32_t leader_id;
+	int32_t replica_cnt, isr_cnt;
+	int32_t partition_cnt;
+	int32_t i, j;
 
 	CHECK_RET(parse_i32(buf, size, &partition_cnt));
 
@@ -2942,12 +2944,19 @@ static int kafka_meta_parse_partition(void **buf, size_t *size,
 
 	for (i = 0; i < partition_cnt; ++i)
 	{
-		if (parse_i16(buf, size, &partition[i]->error) < 0)
+		int16_t error;
+		int32_t index;
+
+		if (parse_i16(buf, size, &error) < 0)
 			break;
 
-		if (parse_i32(buf, size, &partition[i]->partition_index) < 0)
+		partition[i]->error = error;
+
+		if (parse_i32(buf, size, &index) < 0)
 			break;
 
+		partition[i]->partition_index = index;
+
 		if (parse_i32(buf, size, &leader_id) < 0)
 			break;
 
@@ -2962,8 +2971,12 @@ static int kafka_meta_parse_partition(void **buf, size_t *size,
 
 		for (j = 0; j < replica_cnt; ++j)
 		{
-			if (parse_i32(buf, size, &partition[i]->replica_nodes[j]) < 0)
+			int32_t replica_node;
+
+			if (parse_i32(buf, size, &replica_node) < 0)
 				break;
+
+			partition[i]->replica_nodes[j] = replica_node;
 		}
 
 		if (j != replica_cnt)
@@ -2977,8 +2990,12 @@ static int kafka_meta_parse_partition(void **buf, size_t *size,
 
 		for (j = 0; j < isr_cnt; ++j)
 		{
-			if (parse_i32(buf, size, &partition[i]->isr_nodes[j]) < 0)
+			int32_t isr_node;
+
+			if (parse_i32(buf, size, &isr_node) < 0)
 				break;
+
+			partition[i]->isr_nodes[j] = isr_node;
 		}
 
 		if (j != isr_cnt)
@@ -3011,7 +3028,7 @@ static int kafka_meta_parse_topic(void **buf, size_t *size,
 								  KafkaMetaList *meta_list,
 								  KafkaBrokerList *broker_list)
 {
-	int topic_cnt;
+	int32_t topic_cnt;
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 
 	if (topic_cnt < 0)
@@ -3020,9 +3037,9 @@ static int kafka_meta_parse_topic(void **buf, size_t *size,
 		return -1;
 	}
 
-	for (int topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
+	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
-		short error;
+		int16_t error;
 		CHECK_RET(parse_i16(buf, size, &error));
 
 		std::string topic_name;
@@ -3045,7 +3062,7 @@ static int kafka_meta_parse_topic(void **buf, size_t *size,
 
 int KafkaResponse::parse_metadata(void **buf, size_t *size)
 {
-	int throttle_time, controller_id;
+	int32_t throttle_time, controller_id;
 	std::string cluster_id;
 
 	if (this->api_version >= 3)
@@ -3102,12 +3119,12 @@ KafkaToppar *KafkaMessage::find_toppar_by_name(const std::string& topic, int par
 
 int KafkaResponse::parse_produce(void **buf, size_t *size)
 {
-	int topic_cnt;
+	int32_t topic_cnt;
 	std::string topic_name;
-	int partition_cnt;
-	int partition;
+	int32_t partition_cnt;
+	int32_t partition;
 	int64_t base_offset, log_append_time, log_start_offset;
-	int throttle_time;
+	int32_t throttle_time;
 
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 
@@ -3119,12 +3136,12 @@ int KafkaResponse::parse_produce(void **buf, size_t *size)
 
 	int produce_timeout = this->config.get_produce_timeout() * 2;
 
-	for (int topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
+	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
 
 		CHECK_RET(parse_i32(buf, size, &partition_cnt));
-		for (int i = 0; i < partition_cnt; ++i)
+		for (int32_t i = 0; i < partition_cnt; ++i)
 		{
 			CHECK_RET(parse_i32(buf, size, &partition));
 
@@ -3186,20 +3203,20 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 	KafkaToppar *toppar;
 	while ((toppar = this->toppar_list.get_next()) != NULL)
 		toppar->clear_records();
-	int throttle_time;
+	int32_t throttle_time;
 
 	if (this->api_version >= 1)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
 
 	if (this->api_version >= 7)
 	{
-		short error;
-		int sessionid;
+		int16_t error;
+		int32_t sessionid;
 		parse_i16(buf, size, &error);
 		parse_i32(buf, size, &sessionid);
 	}
 
-	int topic_cnt;
+	int32_t topic_cnt;
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 
 	if (topic_cnt < 0)
@@ -3209,14 +3226,14 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 	}
 
 	std::string topic_name;
-	int partition_cnt;
-	int partition;
-	int aborted_cnt;
-	int preferred_read_replica;
-	long long producer_id, first_offset;
+	int32_t partition_cnt;
+	int32_t partition;
+	int32_t aborted_cnt;
+	int32_t preferred_read_replica;
+	int64_t producer_id, first_offset;
 	int64_t high_watermark;
 
-	for (int topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
+	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
 		CHECK_RET(parse_i32(buf, size, &partition_cnt));
@@ -3247,10 +3264,10 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 					CHECK_RET(parse_i64(buf, size, (int64_t *)&ptr->log_start_offset));
 
 				CHECK_RET(parse_i32(buf, size, &aborted_cnt));
-				for (int j = 0; j < aborted_cnt; ++j)
+				for (int32_t j = 0; j < aborted_cnt; ++j)
 				{
-					CHECK_RET(parse_i64(buf, size, (int64_t *)&producer_id));
-					CHECK_RET(parse_i64(buf, size, (int64_t *)&first_offset));
+					CHECK_RET(parse_i64(buf, size, &producer_id));
+					CHECK_RET(parse_i64(buf, size, &first_offset));
 				}
 			}
 
@@ -3274,7 +3291,7 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 
 int KafkaResponse::parse_listoffset(void **buf, size_t *size)
 {
-	int throttle_time, topic_cnt;
+	int32_t throttle_time, topic_cnt;
 
 	if (this->api_version >= 2)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3288,17 +3305,17 @@ int KafkaResponse::parse_listoffset(void **buf, size_t *size)
 	}
 
 	std::string topic_name;
-	int partition_cnt;
-	int partition;
+	int32_t partition_cnt;
+	int32_t partition;
 	int64_t offset_timestamp, offset;
-	int offset_cnt;
+	int32_t offset_cnt;
 
-	for (int topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
+	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
 		CHECK_RET(parse_i32(buf, size, &partition_cnt));
 
-		for (int i = 0; i < partition_cnt; ++i)
+		for (int32_t i = 0; i < partition_cnt; ++i)
 		{
 			CHECK_RET(parse_i32(buf, size, &partition));
 
@@ -3325,7 +3342,7 @@ int KafkaResponse::parse_listoffset(void **buf, size_t *size)
 			else if (this->api_version == 0)
 			{
 				CHECK_RET(parse_i32(buf, size, &offset_cnt));
-				for (int j = 0; j < offset_cnt; ++j)
+				for (int32_t j = 0; j < offset_cnt; ++j)
 				{
 					CHECK_RET(parse_i64(buf, size, &offset));
 					ptr->offset = offset;
@@ -3341,7 +3358,7 @@ int KafkaResponse::parse_listoffset(void **buf, size_t *size)
 
 int KafkaResponse::parse_findcoordinator(void **buf, size_t *size)
 {
-	int throttle_time;
+	int32_t throttle_time;
 
 	if (this->api_version >= 1)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3398,7 +3415,7 @@ static int kafka_cgroup_parse_member(void **buf, size_t *size,
 									 KafkaMetaList *meta_list,
 									 int api_version)
 {
-	int member_cnt = 0;
+	int32_t member_cnt = 0;
 	CHECK_RET(parse_i32(buf, size, &member_cnt));
 
 	if (member_cnt < 0)
@@ -3411,7 +3428,7 @@ static int kafka_cgroup_parse_member(void **buf, size_t *size,
 		return -1;
 
 	kafka_member_t **member = cgroup->get_members();
-	int i;
+	int32_t i;
 
 	for (i = 0; i < member_cnt; ++i)
 	{
@@ -3430,10 +3447,10 @@ static int kafka_cgroup_parse_member(void **buf, size_t *size,
 
 		void *metadata = member[i]->member_metadata;
 		size_t metadata_len = member[i]->member_metadata_len;
-		short version;
-		int topic_cnt;
+		int16_t version;
+		int32_t topic_cnt;
 		std::string topic_name;
-		int j;
+		int32_t j;
 
 		if (parse_i16(&metadata, &metadata_len, &version) < 0)
 			break;
@@ -3471,7 +3488,7 @@ static int kafka_cgroup_parse_member(void **buf, size_t *size,
 
 int KafkaResponse::parse_joingroup(void **buf, size_t *size)
 {
-	int throttle_time;
+	int32_t throttle_time;
 
 	if (this->api_version >= 2)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3495,11 +3512,11 @@ int KafkaMessage::kafka_parse_member_assignment(const char *bbuf, size_t n,
 {
 	void **buf = (void **)&bbuf;
 	size_t *size = &n;
-	int topic_cnt, partition_cnt;
-	short version;
+	int32_t topic_cnt, partition_cnt;
+	int16_t version;
 	struct list_head *pos, *tmp;
 	std::string topic_name;
-	int partition;
+	int32_t partition;
 
 	CHECK_RET(parse_i16(buf, size, &version));
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
@@ -3517,12 +3534,12 @@ int KafkaMessage::kafka_parse_member_assignment(const char *bbuf, size_t n,
 		delete toppar;
 	}
 
-	for (int i = 0; i < topic_cnt; ++i)
+	for (int32_t i = 0; i < topic_cnt; ++i)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
 		CHECK_RET(parse_i32(buf, size, &partition_cnt));
 
-		for (int j = 0; j < partition_cnt; ++j)
+		for (int32_t j = 0; j < partition_cnt; ++j)
 		{
 			CHECK_RET(parse_i32(buf, size, &partition));
 			KafkaToppar *toppar = new KafkaToppar;
@@ -3541,8 +3558,8 @@ int KafkaMessage::kafka_parse_member_assignment(const char *bbuf, size_t n,
 
 int KafkaResponse::parse_syncgroup(void **buf, size_t *size)
 {
-	int throttle_time;
-	short error = 0;
+	int32_t throttle_time;
+	int16_t error;
 	std::string member_assignment;
 
 	if (this->api_version >= 1)
@@ -3564,8 +3581,8 @@ int KafkaResponse::parse_syncgroup(void **buf, size_t *size)
 
 int KafkaResponse::parse_leavegroup(void **buf, size_t *size)
 {
-	int throttle_time;
-	short error = 0;
+	int32_t throttle_time;
+	int16_t error;
 
 	if (this->api_version >= 1)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3578,10 +3595,10 @@ int KafkaResponse::parse_leavegroup(void **buf, size_t *size)
 
 int KafkaResponse::parse_offsetfetch(void **buf, size_t *size)
 {
-	int topic_cnt;
+	int32_t topic_cnt;
 	std::string topic_name;
-	int partition_cnt;
-	int partition;
+	int32_t partition_cnt;
+	int32_t partition;
 
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 
@@ -3591,12 +3608,12 @@ int KafkaResponse::parse_offsetfetch(void **buf, size_t *size)
 		return -1;
 	}
 
-	for (int topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
+	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
 
 		CHECK_RET(parse_i32(buf, size, &partition_cnt));
-		for (int i = 0; i < partition_cnt; ++i)
+		for (int32_t i = 0; i < partition_cnt; ++i)
 		{
 			CHECK_RET(parse_i32(buf, size, &partition));
 			KafkaToppar *toppar = find_toppar_by_name(topic_name, partition,
@@ -3606,8 +3623,8 @@ int KafkaResponse::parse_offsetfetch(void **buf, size_t *size)
 
 			kafka_topic_partition_t *ptr = toppar->get_raw_ptr();
 
-			long offset;
-			CHECK_RET(parse_i64(buf, size, (int64_t *)&offset));
+			int64_t offset;
+			CHECK_RET(parse_i64(buf, size, &offset));
 			if (this->config.get_offset_store() != KAFKA_OFFSET_ASSIGN)
 				ptr->offset = offset;
 
@@ -3621,11 +3638,11 @@ int KafkaResponse::parse_offsetfetch(void **buf, size_t *size)
 
 int KafkaResponse::parse_offsetcommit(void **buf, size_t *size)
 {
-	int throttle_time;
-	int topic_cnt;
+	int32_t throttle_time;
+	int32_t topic_cnt;
 	std::string topic_name;
-	int partition_cnt;
-	int partition;
+	int32_t partition_cnt;
+	int32_t partition;
 
 	if (this->api_version >= 1)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3638,12 +3655,12 @@ int KafkaResponse::parse_offsetcommit(void **buf, size_t *size)
 		return -1;
 	}
 
-	for (int topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
+	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
 		CHECK_RET(parse_i32(buf, size, &partition_cnt));
 
-		for (int i = 0 ; i < partition_cnt; ++i)
+		for (int32_t i = 0 ; i < partition_cnt; ++i)
 		{
 			CHECK_RET(parse_i32(buf, size, &partition));
 			CHECK_RET(parse_i16(buf, size, &this->cgroup.get_raw_ptr()->error));
@@ -3655,8 +3672,8 @@ int KafkaResponse::parse_offsetcommit(void **buf, size_t *size)
 
 int KafkaResponse::parse_heartbeat(void **buf, size_t *size)
 {
-	int throttle_time;
-	short error = 0;
+	int32_t throttle_time;
+	int16_t error;
 
 	if (this->api_version >= 1)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3675,9 +3692,9 @@ static bool kafka_api_version_cmp(const kafka_api_version_t& api_ver1,
 
 int KafkaResponse::parse_apiversions(void **buf, size_t *size)
 {
-	short error;
-	int api_cnt;
-	int throttle_time;
+	int16_t error;
+	int32_t api_cnt;
+	int32_t throttle_time;
 
 	CHECK_RET(parse_i16(buf, size, &error));
 	CHECK_RET(parse_i32(buf, size, &api_cnt));
@@ -3694,7 +3711,7 @@ int KafkaResponse::parse_apiversions(void **buf, size_t *size)
 	this->api->api = (kafka_api_version_t *)p;
 	this->api->elements = api_cnt;
 
-	for (int i = 0; i < api_cnt; ++i)
+	for (int32_t i = 0; i < api_cnt; ++i)
 	{
 		CHECK_RET(parse_i16(buf, size, &this->api->api[i].api_key));
 		CHECK_RET(parse_i16(buf, size, &this->api->api[i].min_ver));
@@ -3712,8 +3729,8 @@ int KafkaResponse::parse_apiversions(void **buf, size_t *size)
 int KafkaResponse::parse_saslhandshake(void **buf, size_t *size)
 {
 	std::string mechanism;
-	int cnt, i;
-	short error;
+	int32_t cnt, i;
+	int16_t error;
 
 	CHECK_RET(parse_i16(buf, size, &error));
 	if (error != 0)
@@ -3751,7 +3768,7 @@ int KafkaResponse::parse_saslhandshake(void **buf, size_t *size)
 int KafkaResponse::parse_saslauthenticate(void **buf, size_t *size)
 {
 	std::string error_message;
-	short error;
+	int16_t error;
 
 	CHECK_RET(parse_i16(buf, size, &error));
 	CHECK_RET(parse_string(buf, size, error_message));
